@@ -1,11 +1,11 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | program/include/iniset.php                                            |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2015, The Roundcube Dev Team                       |
+ | Copyright (C) 2008-2016, The Roundcube Dev Team                       |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -21,7 +21,7 @@
 */
 
 // application constants
-define('RCMAIL_VERSION', '1.1.4');
+define('RCMAIL_VERSION', '1.2.4');
 define('RCMAIL_START', microtime(true));
 
 if (!defined('INSTALL_PATH')) {
@@ -65,18 +65,9 @@ require_once 'Roundcube/bootstrap.php';
 // register autoloader for rcmail app classes
 spl_autoload_register('rcmail_autoload');
 
-// backward compatybility (to be removed)
+// backward compatybility (to be removed in version 1.2.0)
 require_once INSTALL_PATH . 'program/include/bc.php';
 
-// load the UTF-8 portability layers from Patchwork
-// don't load mbstring layer as it conflicts with Roundcube Framework (#1490280)
-if (!function_exists('iconv')) {
-    \Patchwork\Utf8\Bootup::initIconv();
-}
-if (!function_exists('utf8_encode')) {
-    \Patchwork\Utf8\Bootup::initUtf8Encode();
-}
-
 /**
  * PHP5 autoloader routine for dynamic class loading
  */
